#==============================================================================
#==============================================================================
#
#                             myPREDICTING.PUB.R
#
#                         Benoît JAILLARD, January 2017
#     
#                           benoit.jaillard@inra.fr
#
#==============================================================================
#==============================================================================



#==============================================================================
#  Prediction computed
#      amean = by using arithmetic mean
#      byelt = by motif WITH taking into account species contribution
#      by including all the assemblies, even the one to predict
#      for any Function (for instance Fobs)
#==============================================================================
# E[X.Y] = E[X].E[Y] + cov(X,Y)
# par contre G[X.Y] = G[X].G[Y]

predict.amean.byelt <- function(AssMotifs, mOccur, fct) {
  
  setMot  <- unique(AssMotifs)
  
  mfct <- matrix(NA, nrow = length(setMot), ncol = dim(mOccur)[2])
  #  colnames(mfct) <- colnames(mOccur)
  #  rownames(mfct) <- setMot
  
  for (mot in seq_along(setMot)) {
    
    motif  <- setMot[mot]
    indMot <- which(AssMotifs == motif)
    
    for (elt in seq_len(nbElt)) {
      indElt <- which(mOccur[indMot, elt] == 1)
      if (length(indElt) > 0) mfct[motif, elt] <- amean(fct[indMot[indElt]])
    }
  }
  
  
  msk     <- logical(length(fct))
  fctPrd  <- numeric(length(fct))
  sizeAss <- apply(mOccur, MARGIN = 1, FUN = sum)
  
  for (mot in seq_along(setMot)) {
    
    motif  <- setMot[mot]
    indMot <- which(AssMotifs == motif)
    
    if (length(indMot) > 0) {
      setElt <- unique(which((mOccur[indMot, , drop = FALSE] == 1),
                             arr.ind = TRUE)[ , 2])
      
      for (elt in seq_along(setElt)) {
        element <- setElt[elt]
        indElt  <- which(mOccur[indMot, element] == 1)
        
        if (length(indElt) > 0) {
          index <- indMot[indElt]
          fctPrd[index] <- fctPrd[index] + mfct[motif, element]
        }
      }
      
      fctPrd[indMot] <- fctPrd[indMot] / sizeAss[indMot]
      msk[indMot]     <- TRUE
    }
  }
  
  fctPrd[msk == FALSE] <- NA
  
  return(fctPrd)
}



#==============================================================================
#  Prediction computed
#      amean = by using arithmetic mean
#      byelt = by motif WITH taking into account species contribution
#      LOO   = by including all the assemblies, even the one to predict
#      for any Function (for instance Fobs)
#==============================================================================
predict.amean.byelt.LOO <- function(AssMotifs, mOccur, fct) {
  
  mfct <- numeric(dim(mOccur)[2])
  #  names(mfct) <- colnames(mOccur)
  
  fctPrd <- as.numeric(rep(NA, length(fct)))
  
  for (ass in seq_along(AssMotifs)) {
    
    indMot <- setdiff(which(AssMotifs == AssMotifs[ass]), ass)
    setElt <- which(mOccur[ass, ] != 0)
    
    mfct[] <- NA
    if (length(indMot) > 0) {
      for (elt in seq_along(setElt)) {
        element <- setElt[elt]
        indElt  <- which(mOccur[indMot, element] == 1)
        if (length(indElt) > 0) mfct[element] <- amean(fct[indMot[indElt]])
      }
      
      fctPrd[ass] <- amean(mfct[setElt])
    }
  }
  
  return(fctPrd)
}



#==============================================================================
#  Prediction computed
#      gmean = by using geometric mean
#      byelt = by motif WITH taking into account species contribution
#      by including all the assemblies, even the one to predict
#      for any Fonction fct (Fobs for instance)
#==============================================================================
predict.gmean.byelt <- function(AssMotifs, mOccur, fct) {

  setMot <- unique(AssMotifs)

  mfct <- matrix(NA, nrow = length(setMot), ncol = dim(mOccur)[2])
#  colnames(mfct) <- colnames(mOccur)
#  rownames(mfct) <- setMot

  for (mot in seq_along(setMot)) {
    
    motif  <- setMot[mot]
    indMot <- which(AssMotifs == motif)
    setElt <- unique(which((mOccur[indMot, , drop = FALSE] == 1),
                           arr.ind = TRUE)[ , 2])

    for (elt in seq_along(setElt)) {
      element <- setElt[elt]
      indElt  <- which(mOccur[indMot, element] == 1)
      mfct[motif, element] <- gmean(fct[indMot[indElt]])
    }
  }


  msk      <- logical(length(fct))
  fctPrd   <- numeric(length(fct))
  fctPrd[] <- 1

  sizeAss  <- apply(mOccur, MARGIN = 1, FUN = sum)

  for (mot in seq_along(setMot)) {
    
    motif  <- setMot[mot]
    indMot <- which(AssMotifs == motif)
    setElt <- unique(which((mOccur[indMot, , drop = FALSE] == 1),
                            arr.ind = TRUE)[ , 2])

    if (length(indMot) > 0) {
      for (elt in seq_along(setElt)) {
        element <- setElt[elt]
        indElt  <- which(mOccur[indMot, element] == 1)

        if (length(indElt) > 0) {
          index <- indMot[indElt]
          fctPrd[index] <- fctPrd[index] * mfct[motif, element]
        }
      }

      fctPrd[indMot] <- fctPrd[indMot] ^ (1/sizeAss[indMot])
      msk[indMot]    <- TRUE
    }
  }

  fctPrd[msk == FALSE] <- NA

  return(fctPrd)
}



#==============================================================================
#  Prediction computed
#      gmean = by using geometric mean
#      byelt = by motif WITH taking into account species contribution
#      LOO = by excluding the assembly to predict    (Leave One Out)
#      for any Fonction fct (Fobs for instance)
#==============================================================================
predict.gmean.byelt.LOO <- function(AssMotifs, mOccur, fct) {

  mfct    <- numeric(dim(mOccur)[2])
#  names(mfct) <- colnames(mOccur)

  fctPrd <- as.numeric(rep(NA, length(fct)))

  for (ass in seq_along(AssMotifs)) {
    indMot <- setdiff(which(AssMotifs == AssMotifs[ass]), ass)

    setElt <- which(mOccur[ass, ] != 0)

    mfct[] <- NA
    if (length(indMot) > 0) {
      for (elt in seq_along(setElt)) {
        element <- setElt[elt]
        indElt  <- which(mOccur[indMot, element] == 1)
        if (length(indElt) > 0) mfct[element] <- gmean(fct[indMot[indElt]])
      }

      fctPrd[ass] <- gmean(mfct[setElt])
    }
  }

  return(fctPrd)
}



#==============================================================================
#  Compute and Plot
#         the matrix 3 x nbclusters of R2cal, R2prd, and missing values
#==============================================================================
compute.mStats <- function(mCal, mPrd, fct) {
  
  nbclMax <- dim(mCal)[1]
  
  tmp <- c("missing", "R2cal", "R2prd", "pcal", "pprd")
  
  mStats           <- matrix(0, nrow = nbclMax, ncol = length(tmp))
  rownames(mStats) <- seq(1, nbclMax)
  colnames(mStats) <- tmp

  nbcl <- 1
  mStats[nbcl, "missing"] <- 
    length(na.action(na.omit(mPrd[nbcl, ]))) / length(mCal[nbcl, ])
  mStats[nbcl, "R2cal"] <- R2mse(mCal[nbcl, ], fct)
  mStats[nbcl, "R2prd"] <- R2mse(mPrd[nbcl,],  fct)

  for (nbcl in 2:nbclMax) {
    mStats[nbcl, "missing"] <- 
      length(na.action(na.omit(mPrd[nbcl, ]))) / length(mCal[nbcl, ])
    mStats[nbcl, "R2cal"] <- R2mse(mCal[nbcl, ], fct)
    mStats[nbcl, "R2prd"] <- R2mse(mPrd[nbcl,],  fct)
    mStats[nbcl, "pcal"]  <- pmse( mCal[nbcl, ], fct, nbcl)
    mStats[nbcl, "pprd"]  <- pmse( mPrd[nbcl,],  fct, nbcl)
  }
  
  return(mStats)
}



#==============================================================================
#==============================================================================
#==============================================================================
#==============================================================================
#==============================================================================
#==============================================================================

#==============================================================================
#
#  Plot the matrix 3 x nbclusters of R2cal, R2prd, and missing values
#
#==============================================================================
plot.mStats <- function(mStats, nbElt, titre = "") {

  nbclMax <- dim(mStats)[1]

  #------------------------------------------------------------ ---------------
  # plot a first graph without any pvalues
  #----------------------------------------------------------- ----------------
  plot(x = seq_len(nbclMax), y = mStats[ ,"R2cal"],
       xlim = c(1, nbElt),   ylim = c(0,1),
       type = "n", pch = 1, cex = 2, tck = 0.02, las = 1,
       bg = "white", col = "black",
       ylab = "R2cal (pvalue)/ R2prd (pvalue, pdif)",
       xlab = "number of clusters",
       main = titre)

  # plot R2calibration
  points(x = seq_len(nbclMax), y = mStats[ ,"R2cal"],
         type = "b", pch = 1, cex = 2, bg = "white", col = "black")

  # plot R2prediction with pvalues
  points(x = seq_len(nbclMax), y = mStats[ ,"R2prd"],
         type = "b", pch = 1, cex = 2, bg = "white", col = "red3")


  #-------------------------------------------------------------- -------------
  # plot a second graph without any pvalues
  #-------------------------------------------------------------- -------------
  plot(x = seq_len(nbclMax), y = mStats[ ,"R2cal"],
       xlim = c(1, nbElt), ylim = c(0,1),
       type = "n", pch = 1, cex = 2, tck = 0.02, las = 1,
       bg = "white", col = "black",
       ylab = "Predicting ratio",
       xlab = "number of clusters",
       main = titre)

  # plot predictiong ratio
  points(x = seq_len(nbclMax), y = 1 - mStats[ ,"missing"],
         type = "b", pch = 0, cex = 2, bg = "white", col = "blue")

}



#==============================================================================
#  Plot Predictions vs Observations
#
#  Inputs : Fprd : vector of modelled or predicted values
#           Fobs : vector of observed values
#           AssMotifs : vector of motifs of which belong the assemblies
#
#  Options : titre : main titre of Figure
#            opt.reg : technical information on the quality of the regression
#            opt.aov : variance analysis of assembly function by motif
#            pvalue : threshold for the aov analysis
#
#==============================================================================
plot.prediction.simple <- function(Fprd, Fobs, AssMotifs, nbcl,
                                   titre    = "",
                                   opt.mean = "amean",
                                   opt.aov  = FALSE,
                                   pvalue   = 0.025,
                                   opt.reg  = FALSE) {
  
  #----------------------------------------------------------------------------
  # Check the inputs and plot the figure
  #----------------------------------------------------------------------------
  nbAss <- length(Fobs)
  xylim <- range(Fobs)
  if (opt.mean == "amean") tmp <- amean(Fobs)
  if (opt.mean == "gmean") tmp <- gmean(Fobs)
  
  index <- na.action(na.omit(Fprd))
  if (length(index) == nbAss) stop("the vector Fprd cannot be null")
  
  if (length(index) > 0) {
    Fprd      <- Fprd[-index]
    Fobs      <- Fobs[-index]
    AssMotifs <- AssMotifs[-index]
  }
  
  plot(x = Fobs, xlab = "Observations", xlim = xylim,
       y = Fprd, ylab = "Predictions",  ylim = xylim,
       main = titre,
       type = "n", tck = 0.02, las = 1)
  
  abline(v = tmp, h = tmp, lty = "dotted", col = "blue")
  
  lines(x = xylim, y = xylim, lty = "solid", col = "red")
  
  points(x = Fobs, y = Fprd,
         pch = figures[AssMotifs], col = couleurs[AssMotifs],
         bg = "white", cex = 2)
  
  
  #----------------------------------------------------------------------------
  # Adding of various useful informations
  #----------------------------------------------------------------------------
  width <- xylim[2] - xylim[1]
  posX1 <- xylim[1] + 0.75*width
  posX2 <- xylim[1] + 1.02*width
  posX3 <- xylim[1] - 0.02*width
  posX4 <- xylim[1] + 0.02*width
  posY1 <- xylim[1] + 0.05*width
  posY2 <- xylim[1]
  posY3 <- xylim[1] + 1.02*width
  posY4 <- xylim[1] + 0.98*width
  
  
  # predicting ratio
  tmp <- paste("predicting = ", length(na.omit(Fprd)), "/", nbAss, sep = "")
  text(x = posX1, y = posY1, labels = tmp, col = "red")
  
  # R2 value
  if ((nbAss - length(index)) > 1) {
    tmp <- paste("R2mse = ", signif(R2mse(Fprd, Fobs), digits = 3), sep = "")
    text(x = posX1, y = posY2, labels = tmp, col = "red")
  }
  
  # Mean number of clusters
  text(paste("N = ", signif(nbcl, digits = 3), sep = ""),
       x = posX4, y = posY4, col = "red")
  
  #----------------------------------------------------------------------------
  # Optional informations
  #----------------------------------------------------------------------------
  # option variance analysis
  if (opt.aov == TRUE) {
    setMot <- unique(AssMotifs)
    
    text(x = posX2, y = posY3, labels = "pred", col = "black")
    
    test    <- test.posthoc(Fprd, AssMotifs, pvalue)
    if (is.list(test))
      for (mot in seq_along(setMot)) {
        motif <- setMot[mot]
        mt    <- as.matrix(test$groups)
        index <- which(as.integer(mt[,"trt"]) == motif)
        text(x = posX2, y = as.numeric(mt[index, "means"]),
             labels = as.character(mt[index, "M"]),
             col =  couleurs[motif], font = 3)
      }
  }
  
  # option regression
  if (opt.reg == TRUE) plot(model)
  
}



#==============================================================================
#  Plot Predictions vs Observations
#==============================================================================
plot.prediction.LOO <- function(Fcal, Fprd, Fobs, AssMotifs, nbcl,
                                titre    = "",
                                opt.mean = "amean",
                                opt.aov  = FALSE,
                                pvalue   = 0.025,
                                opt.reg  = FALSE)
{

  #----------------------------------------------------------------------------
  # Check the inputs and plot the figure
  #----------------------------------------------------------------------------
  nbAss <- length(Fobs)
  xylim <- range(Fobs)
  if (opt.mean == "amean") tmp <- amean(Fobs)
  if (opt.mean == "gmean") tmp <- gmean(Fobs)
  
  index <- na.action(na.omit(Fprd))
  if (length(index) == nbAss) return(FALSE)

  if (length(index) > 0) {
    Fprd      <- Fprd[-index]
    Fcal      <- Fcal[-index]
    Fobs      <- Fobs[-index]
    AssMotifs <- AssMotifs[-index]
  }

  plot(x = Fobs, xlab = "Observations", xlim = xylim,
       y = Fcal, ylab = "Predictions",  ylim = xylim,
       main = titre,
       las = 1, type = "n", tck = 0.02)

  for (elt in seq_along(Fobs))
    lines(x = c(Fobs[elt], Fobs[elt]), y = c(Fprd[elt], Fcal[elt]),
          col = couleurs[AssMotifs[elt]], lty = "solid")

  abline(v = tmp, h = tmp, lty = "dotted", col = "blue")

  lines(x = xylim, y = xylim, lty = "solid", col = "red")

  points(x = Fobs, y = Fcal,
         pch = figures[AssMotifs], col = couleurs[AssMotifs],
         bg = "white", cex = 2)


  #----------------------------------------------------------------------------
  # Adding of various useful informations
  #----------------------------------------------------------------------------
  width <- xylim[2] - xylim[1]
  posX1 <- xylim[1] + 0.75*width
  posX2 <- xylim[1] + 1.02*width
  posX3 <- xylim[1] - 0.02*width
  posX4 <- xylim[1] + 0.02*width
  posY1 <- xylim[1] + 0.05*width
  posY2 <- xylim[1]
  posY3 <- xylim[1] + 1.02*width
  posY4 <- xylim[1] + 0.98*width


  # Predicting assemblies
  tmp <- paste("predicting = ", length(na.omit(Fprd)), "/", nbAss, sep = "")
  text(x = posX1,  y = posY1, labels = tmp, col = "red")

  # R2 value
  if ((nbAss - length(index)) > 1) {
    tmp <- paste("R2cal = ", signif(R2mse(Fcal, Fobs), digits = 3),
                 "   R2prd = ", signif(R2mse(Fprd, Fobs), digits = 3), sep = "")
    text(x = posX1, y = posY2, labels = tmp, col = "red")
  }

  # Mean number of clusters
  text(paste("N = ", signif(nbcl, digits = 3), sep = ""),
       x = posX4, y = posY4, col = "red")

  #----------------------------------------------------------------------------
  # Optional informations
  #----------------------------------------------------------------------------
  # option variance analysis
  if (opt.aov == TRUE) {
    setMot <- unique(AssMotifs)

    text(x = posX3, y = posY3, labels = "cal", col = "black")
    test <- test.posthoc(Fcal, AssMotifs, pvalue)

    if (is.list(test))
      for (mot in seq_along(setMot)) {
        motif <- setMot[mot]
        mt    <- as.matrix(test$groups)
        index <- which(as.integer(mt[,"trt"]) == motif)
        text(x = posX3, y = as.numeric(mt[index, "means"]),
             labels = as.character(mt[index, "M"]),
             col =  couleurs[motif], font = 3)
      }

    text(x = posX2, y = posY3, labels = "prd", col = "black")

    test <- test.posthoc(Fprd, AssMotifs, pvalue)
    if (is.list(test))
      for (mot in seq_along(setMot)) {
        motif <- setMot[mot]
        mt    <- as.matrix(test$groups)
        index <- which(as.integer(mt[,"trt"]) == motif)
        text(x = posX2, y = as.numeric(mt[index, "means"]),
             labels = as.character(mt[index, "M"]),
             col =  couleurs[motif], font = 3)
      }
  }

}




#==============================================================================
#  Plot all the necessary predicting figures
#==============================================================================
predict.function <- function(mAssMotifs, mOccur, fct,
                             titre    = "",
# options for computing
                             opt.mean = "amean",
# options for plotting
                             opt.stats = FALSE,
                             opt.cal  = FALSE,
                             opt.prd  = FALSE,
                             opt.aov  = FALSE, pvalue = 0.025,
                             opt.reg  = FALSE) 
{

  #----------------------------------------------------------------------------
  #  Main calculations of Cal, Prd, R2cal and R2prd
  #----------------------------------------------------------------------------
  nbElt     <- dim(mOccur)[2]
  nbAss     <- length(fct)

  # Affect the motifs to assemblages
  nbclMax    <- dim(mAssMotifs)[1]
  
  # Compute the cross-validation predictions
  mCal <- mPrd <- matrix(NA, nrow = nbclMax, ncol = nbAss)
  rownames(mCal) <- rownames(mPrd) <- seq_len(nbclMax)

  for (nbcl in seq_len(nbclMax)) {
    AssMotifs    <- mAssMotifs[nbcl, ]
    if (opt.mean == "amean") {
      mCal[nbcl, ] <- predict.amean.byelt(AssMotifs, mOccur, fct)
      mPrd[nbcl, ] <- predict.amean.byelt.LOO(AssMotifs, mOccur, fct)
    } else {
      mCal[nbcl, ] <- predict.gmean.byelt(AssMotifs, mOccur, fct)
      mPrd[nbcl, ] <- predict.gmean.byelt.LOO(AssMotifs, mOccur, fct)
    }
  }
  mStats <- compute.mStats(mCal, mPrd, fct)
  

  #----------------------------------------------------------------------------
  #  Main plotting
  #----------------------------------------------------------------------------

  # Plot the Best prediction
  if (opt.stats == TRUE)
    plot.mStats(mStats, nbElt,
             paste("R2 calibration and prediction:", titre,
                   opt.mean, sep = " ") )


  #----------------------------------------------------------------------------
  #  Optional plotting
  #----------------------------------------------------------------------------
  # Plot Calibrations for all the number of clusters
  if (opt.cal == TRUE)
    for (nbcl in seq_len(nbclMax)) {
      titre2 <- paste(titre, "Calibration with", nbcl, "clusters", sep = " ")
      plot.prediction.simple(mCal[nbcl, ], fct,
                             mAssMotifs[nbcl,], nbcl,
                             opt.mean = opt.mean,
                             opt.aov = opt.aov, pvalue = pvalue,
                             titre = titre2)
  }


  # Plot Predictions for all the number of classes
  if (opt.prd == TRUE)
    for (nbcl in seq_len(nbclMax))
      if (length(na.omit(mPrd[nbcl, ])) > 0) {
        titre2 <- paste(titre, "Prediction with", nbcl, "clusters", sep = " ")
        plot.prediction.LOO(mCal[nbcl, ], mPrd[nbcl, ], fct,
                            mAssMotifs[nbcl,], nbcl,
                            opt.mean = opt.mean,
                            opt.aov  = opt.aov, pvalue = pvalue,
                            titre = titre2)
  }


  #----------------------------------------------------------------------------
  #  Outputs
  #----------------------------------------------------------------------------
  res <- list(mCal, mPrd, mStats)
  names(res) <- c("mCal", "mPrd", "mStats")

  return(res)
}



#==============================================================================
#  Plot all the necessary predicting figures
#==============================================================================
predict.twin <- function(mAssMotifs, mOccur, alpha, beta, fscale = 1, 
                         titre     = "",
# options for plotting
                         opt.stats = FALSE,
                         opt.cal   = FALSE,
                         opt.prd   = FALSE,
                         opt.aov  = FALSE, pvalue = 0.025,
                         opt.reg  = FALSE) 
{
  
  #----------------------------------------------------------------------------
  #  Main calculations of Cal, Prd, R2cal and R2prd
  #----------------------------------------------------------------------------
  nbElt   <- dim(mOccur)[2]
  nbAss   <- length(alpha)
  nbclMax <- dim(mAssMotifs)[1]

  # Compute the cross-validation predictions
  mCal <- mPrd <- matrix(NA, nrow = nbclMax, ncol = nbAss)
  rownames(mCal) <- rownames(mPrd) <- seq_len(nbclMax)

  for (nbcl in seq_len(nbclMax)) {
    AssMotifs    <- mAssMotifs[nbcl, ]
    mCal[nbcl, ] <- predict.gmean.byelt(AssMotifs, mOccur, alpha) * 
                    predict.amean.byelt(AssMotifs, mOccur, beta) *
                    fscale
    mPrd[nbcl, ] <- predict.gmean.byelt.LOO(AssMotifs, mOccur, alpha) * 
                    predict.amean.byelt.LOO(AssMotifs, mOccur, beta) *
                    fscale
  }
  fct <- alpha * beta * fscale
  mStats <- compute.mStats(mCal, mPrd, fct)
  

  #----------------------------------------------------------------------------
  #  Main plotting
  #----------------------------------------------------------------------------

  # Plot the Best prediction
  if (opt.stats == TRUE) {

    titre2 <- paste("R2 calibration and prediction:", titre, sep = " ")
    plot.mStats(mStats, nbElt, titre2)

  }


  #----------------------------------------------------------------------------
  #  Optional plotting
  #----------------------------------------------------------------------------
  # Plot Calibrations for all the number of clusters
  if (opt.cal == TRUE)
    for (nbcl in seq_len(nbclMax)) {
      titre2 <- paste(titre, "Calibration with", nbcl, "clusters", sep = " ")
      plot.prediction.simple(mCal[nbcl, ], fct,
                             mAssMotifs[nbcl,], nbcl,
                             opt.aov = opt.aov, pvalue = pvalue,
                             titre = titre2)
    }


  # Plot Predictions for all the number of classes
  if (opt.prd == TRUE)
    for (nbcl in seq_len(nbclMax))
      if (length(na.omit(mPrd[nbcl, ])) > 0) {
        titre2 <- paste(titre, "Prediction with", nbcl, "clusters", sep = " ")
        plot.prediction.LOO(mCal[nbcl, ], mPrd[nbcl, ], fct,
                            mAssMotifs[nbcl,], nbcl,
                            opt.aov = opt.aov, pvalue = pvalue,
                            titre = titre2)
      }


  #----------------------------------------------------------------------------
  #  Outputs
  #----------------------------------------------------------------------------
  res <- list(mCal, mPrd, mStats)
  names(res) <- c("mCal", "mPrd", "mStats")

  return(res)
}



#==============================================================================
#==============================================================================
#
#                   END of FILE myPREDICTING.R
#
#==============================================================================
#==============================================================================
