#==============================================================================
#==============================================================================
# 
#                    myBEGINNING.SCRIPT.R
#                  
#                  set of instructions to run at first
#                   
#                   Benoît JAILLARD, January 2017
#                   
#                      benoit.jaillard@inra.fr
#
#==============================================================================
#==============================================================================


#==============================================================================
#  loading of usefull libraries and local sources
#==============================================================================


#==============================================================================
# Multiplicative decomposition of assembly performances
#==============================================================================
res    <- multiplicative.decomposition(mOccur, Fobs, rm.mono = TRUE)
size   <- res$size
mOccur <- res$mOccur
Fobs   <- res$Fobs
Fmono  <- res$Fmono
Fscale <- res$Fscale

fobs   <- Fobs/Fscale
alpha  <- res$alpha
beta   <- res$beta
trans  <- res$trans
nbAss  <- dim(mOccur)[1]
nbElt  <- dim(mOccur)[2]


nbclassesMax <- nbElt



#==============================================================================
# Computation of the hierarchical tree
#==============================================================================
alphaElt <- betaElt <- fobsElt <-
  sdalphaElt <- sdbetaElt <- sdfobsElt <- numeric(nbElt)
names(alphaElt) <- names(betaElt) <- colnames(mOccur)

for (elt in seq_len(nbElt)) {
  index           <- which(mOccur[,elt] == 1)

  alphaElt[elt]   <- gmean(alpha[index])
  sdalphaElt[elt] <- gsd(alpha[index])

  betaElt[elt]    <- amean(beta[index])
  sdbetaElt[elt]  <- asd(beta[index])
}
fobsElt   <- alphaElt * betaElt
sdfobsElt <- (sdalphaElt / alphaElt + sdbetaElt / betaElt) * fobsElt


dot       <- cbind(alphaElt, betaElt)
dot.dist  <- dist(x = dot, method = "euclidean", diag = FALSE, upper = FALSE)
dot.arbre <- hclust(d = dot.dist, method = "ward.D")



# Record the species clustering and associated assembly motifs
mAffectElt <- matrix(0, nrow = nbElt, ncol = nbElt)
colnames(mAffectElt) <- colnames(mOccur)
rownames(mAffectElt) <- c(1:nbElt)

mAssMotifs <- matrix(0, nrow = nbElt, ncol = nbAss)
colnames(mAssMotifs) <- rownames(mOccur)
rownames(mAssMotifs) <- c(1:nbElt)

for (nbclasses in seq_len(nbclassesMax)) {
  affectElt               <- cutree(dot.arbre, nbclasses)
  mAffectElt[nbclasses, ] <- affectElt
  mAssMotifs[nbclasses, ] <- affect.motifs(affectElt, mOccur)
}



#==============================================================================
#==============================================================================
#
#     END of FILE
#
#==============================================================================
#==============================================================================
