#==============================================================================
#==============================================================================
#
#                        nyPREDICTING.SCRIPT.R
#                        
#                        Benoît JAILLARD, January 2017
#                        
#                          benoit.jaillard@inra.fr
#
#==============================================================================
#==============================================================================


#==============================================================================
# Clear Memory, load files and global variables
#==============================================================================


#==============================================================================
#==============================================================================
#
#   SCRIPT for Predicting performance and Plotting the Results
#
#==============================================================================
#==============================================================================

opt.R2   <- TRUE
opt.cal  <- FALSE
opt.prd  <- FALSE
opt.glb  <- FALSE
opt.pub  <- FALSE
opt.aov  <- FALSE

opt.size <- FALSE
opt.gf   <- FALSE
opt.box  <- FALSE
opt.all  <- FALSE


opt.size <- TRUE
opt.gf   <- TRUE
#opt.box <- TRUE
#opt.all <- TRUE


opta <- "gmean"
optb <- "amean"

#==============================================================================
# Plot the predicted ALPHA, BETA and FOBS by ASSEMBLY MOTIFS
#==============================================================================

res.alpha <- predict.function(mAssMotifs, mOccur, alpha,
                              opt.mean = "gmean",
                              opt.stats = TRUE,
                              opt.cal = TRUE,
                              opt.prd = TRUE,
                              opt.aov = TRUE,
                              pvalue = pvalue,
                              titre = "alpha")


res.beta <- predict.function(mAssMotifs, mOccur, beta,
                             opt.mean = "amean",
                             opt.stats = TRUE,
                             opt.cal = TRUE,
                             opt.prd = TRUE,
                             opt.aov = TRUE,
                             pvalue = pvalue,
                             titre = "beta")


res.abFobs <- predict.twin(mAssMotifs, mOccur, alpha, beta,
                              fscale = Fscale,
                              opt.stats = TRUE,
                              opt.cal = TRUE,
                              opt.prd = TRUE,
                              opt.aov = TRUE,
                              pvalue = pvalue,
                              titre = "abc = Fobs")




#==============================================================================
# Predicting the assembly performance with THE ASSEMBLY SIZE
#==============================================================================
if (opt.size == TRUE) {

  size       <- apply(mOccur, MARGIN = 1, FUN = sum)
  nbsize     <- length(unique(size))

  size.AssMotifs <- matrix(1, nrow = nbsize, ncol = nbAss)
  size.AssMotifs[nbsize, ] <- compact.index(size)

  titre <- c("ASSEMBLY SIZE: alpha using gSpeciesByMotif ")
  size.alpha <- predict.function(size.AssMotifs, mOccur, alpha,
                                 opt.mean = "gmean",
                                 opt.stats = TRUE,
                                 opt.cal = TRUE,
                                 opt.prd = TRUE,
                                 opt.aov = TRUE,
                                 pvalue = pvalue,
                                 titre = titre)

  titre <- c("ASSEMBLY SIZE: beta using aSpeciesByMotif ")
  size.beta <- predict.function(size.AssMotifs, mOccur, beta,
                                opt.mean = "amean",
                                opt.stats = TRUE,
                                opt.cal = TRUE,
                                opt.prd = TRUE,
                                opt.aov = TRUE,
                                pvalue = pvalue,
                                titre = titre)
  
  titre <- c("ASSEMBLY SIZE: Fobs using g/aSpeciesByMotif ")
  size.Fobs <- predict.twin(size.AssMotifs, mOccur, alpha, beta,
                            fscale = Fscale,
                            opt.stats = TRUE,
                            opt.cal = TRUE,
                            opt.prd = TRUE,
                            opt.aov = TRUE,
                            pvalue = pvalue,
                            titre = titre)
}



#==============================================================================
# Predicting the assembly performances with A PRIORI FUNCTIONAL GROUPS
#==============================================================================
if ((opt.gf == TRUE) & (length(unique(gf)) > 1)) {

  nbgf <- length(unique(gf))

  gf.AssMotifs         <- matrix(1, nrow = nbgf, ncol = nbAss)
  gf.AssMotifs[nbgf, ] <- affect.motifs(gf, mOccur)

  titre <- c("FUNCTIONAL GROUPS: alpha using gSpeciesByMotif")
  gf.alpha <- predict.function(gf.AssMotifs, mOccur, alpha,
                               opt.mean = opta,
                               opt.stats = TRUE,
                               opt.cal = TRUE,
                               opt.prd = TRUE,
                               opt.aov = TRUE,
                               pvalue = pvalue,
                               titre = titre)

  titre <- c("FUNCTIONAL GROUPS: beta using aSpeciesByMotif ")
  gf.beta <- predict.function(gf.AssMotifs, mOccur, beta,
                              opt.mean = optb,
                              opt.stats = TRUE,
                              opt.cal = TRUE,
                              opt.prd = TRUE,
                              opt.aov = TRUE,
                              pvalue = pvalue,
                              titre = titre)

  titre <- c("FUNCTIONAL GROUPS: Fobs using g/aSpeciesByMotif ")
  gf.Fobs <- predict.twin(gf.AssMotifs, mOccur, alpha, beta,
                          fscale = Fscale,
                          opt.stats = TRUE,
                          opt.cal = TRUE,
                          opt.prd = TRUE,
                          opt.aov = TRUE,
                          pvalue = pvalue,
                          titre = titre)
}


#==============================================================================
#==============================================================================
# END of SCRIPT
#==============================================================================
#==============================================================================



