#==============================================================================
#==============================================================================
#
#     mySEPARATING.R = set of instructions to analyse a diversity dataset
#
#   SCRIPT for Separating the Interaction effect from the Composition effect
#
#                      Benoît JAILLARD, January 2017
#
#                         benoit.jaillard@inra.fr
#
#==============================================================================
#==============================================================================



#==============================================================================
# Parameters for graph plotting
#==============================================================================
opt.size <- TRUE
opt.gf   <- TRUE
opt.log  <- FALSE

pvalue  <- 0.05

higth <- max(alpha) - min(alpha)
width <- max(beta)  - min(beta)
posX1 <- min(beta)  - 0.02*width
posX2 <- min(beta)  + 1.02*width
posY1 <- min(alpha) - 0.02*higth
posY2 <- min(alpha) + 1.02*higth



#==============================================================================
# Plot Fmono
#==============================================================================
index <- sort(Fmono, decreasing = TRUE, index.return = TRUE)

plot(x = seq_len(nbElt), xlab = "elements",
     y = Fmono[index$ix], ylab = "performance in monoculture",
     type = "p", las = 1, cex = 2, tck = 0.02)

axis(seq_len(nbElt), colnames(mOccur)[index$ix],
     side = 3, tick = FALSE, las = 2, col = "black")



#==============================================================================
# Plot beta versus Fmono
#==============================================================================
ymin <- min(betaElt - sdbetaElt)
ymax <- max(betaElt + sdbetaElt)
fct  <- Fmono

plot(x = fct,   xlab = "performance in monoculture",
     y = betaElt, ylab = "beta", ylim = c(ymin, ymax),
     type = "n", las = 1, cex = 2, tck = 0.02)

arrows(x0 = fct, y0 = betaElt - sdbetaElt,
       x1 = fct, y1 = betaElt + sdbetaElt,
       col = "black", length = 0.1, angle = 90, code = 3,
       lwd = 1, lty = "solid")

points(x = fct,  y = betaElt,
       type = "p", las = 1, cex = 2, bg = "white")

R2 <- cor2(betaElt, fct)^2

text(x = min(fct) + 3/4*(max(fct) - min(fct)), y = min(betaElt),
       labels = paste("R2 = ", signif(R2, digits = 3), sep = ""),
       pos = 3, col = "blue")


#==============================================================================
# Plot alpha
#==============================================================================
titre <- "alpha"

ghisto(alpha, titre, scale.log = FALSE)
plot.alphaVSbeta(x = fobs, xlab = "fobs",
                 y = alpha, ylab = titre,
                 titre = titre, scale.log = FALSE)

plot.alphaVSbeta(y = fobs, ylab = "fobs",
                 x = alpha, xlab = titre,
                 titre = titre, scale.log = FALSE)



#==============================================================================
# Plot beta
#==============================================================================
titre <- "beta"

ghisto(beta, titre, scale.log = FALSE)
plot.alphaVSbeta(x = fobs, xlab = "fobs",
                 y = beta, ylab = titre,
                 titre = titre, scale.log = FALSE)

plot.alphaVSbeta(y = fobs, ylab = "fobs",
                 x = beta, xlab = titre,
                 titre = titre, scale.log = FALSE)



#==============================================================================
# Plot log2(alpha) vs log2(beta)
#==============================================================================
plot.alphaVSbeta(x = beta,  xlab = "beta",
                 y = alpha, ylab = "alpha",
                 titre = "alpha vs beta", scale.log = FALSE)



#==============================================================================
# Plot the mean performance of assemblages containing a given element
#==============================================================================
for (elt in seq_len(nbElt)) {
  index <- which(mOccur[,elt] == 1)

  plot(x = beta,  xlab = "beta",
       y = alpha, ylab = "alpha",
       main = paste("Element = ", colnames(mOccur)[elt], sep = ""),
       type = "n", tck = 0.02, las = 1)
  points(x = beta[-index], y = alpha[-index],
         type = "p", pch = figures[1], col = "black", bg = "white", cex = 2)
  points(x = beta[index], y = alpha[index],
         type = "p", pch = figures[2], col = "blue", bg = "white", cex = 2)
  points.sd(x = beta[index], y = alpha[index],
            figure = figures[2], couleur = "blue", nom = colnames(mOccur)[elt],
            opt.std = TRUE, scale.log = FALSE)
  abline(h = 0, v = 0, col = "black")
  abline(h = gmean(alpha), v = mean(beta), col = "blue")
}



#==============================================================================
# Plot all species without standard deviation WITH RAW DATA
#==============================================================================
plot(x = beta,  xlab = "beta",
     y = alpha, ylab = "alpha",
     main = "All elements",
     pch = figures[1], col = "black", bg = "white",
     type = "p", cex = 2, tck = 0.02, las = 1)

abline(h = 0, v = 0, col = "black")
abline(h = gmean(alpha), v = mean(beta), col = "blue")

for (elt in 1:nbElt) {
  index <- which(mOccur[,elt] == 1)
  points.sd(x = beta[index], y = alpha[index],
            figure = figures[elt], couleur = couleurs[elt],
            nom = colnames(mOccur)[elt], opt.std = FALSE, scale.log = FALSE)
}



#==============================================================================
# Plot all species without standard deviation WITH RAW DATA,
#     enlarged and focused on species centroids
#==============================================================================
plot(x = beta,  xlab = "beta",
     y = alpha, ylab = "alpha",
     xlim = c(0.9, 1.225), ylim = c(1.4, 1.6),  # for Langenheder's data
#     xlim = c(0.76, 1.4), ylim = c(1.8, 3.6),  # for Tilman's data
     main = "All elements",
     pch = figures[1], col = "black", bg = "white",
     type = "n", cex = 2, tck = 0.02, las = 1)

abline(h = 0, v = 0, col = "black")
abline(h = gmean(alpha), v = mean(beta), col = "blue")

for (elt in 1:nbElt) {
  index <- which(mOccur[,elt] == 1)
  points.sd(x = beta[index], y = alpha[index],
            figure = figures[elt], couleur = couleurs[elt],
            nom = colnames(mOccur)[elt], opt.std = FALSE, scale.log = FALSE)
}



#==============================================================================
# Plot all species without standard deviation WITH RAW DATA WITH SD
#==============================================================================
plot(x = beta,  xlab = "beta",
     y = alpha, ylab = "alpha",
     main = "All elements",
     pch = figures[1], col = "black", bg = "white",
     type = "p", cex = 2, tck = 0.02, las = 1)

abline(h = 0, v = 0, col = "black")
abline(h = gmean(alpha), v = mean(beta), col = "blue")

for (elt in 1:nbElt) {
  index <- which(mOccur[,elt] == 1)
  points.sd(x = beta[index], y = alpha[index],
            figure = figures[elt], couleur = couleurs[elt],
            nom = colnames(mOccur)[elt], opt.std = TRUE, scale.log = FALSE)
}



#==============================================================================
# Plot all species without standard deviation WITHOUT RAW DATA
#==============================================================================
plot(x = beta,  xlab = "beta",
     y = alpha, ylab = "alpha",
     #     xlim = c(-0.2,0.4), ylim = c(0.4,0.7),
     main = "All elements",
     type = "n", bg = "white", tck = 0.02, las = 1)
abline(h = 0, v = 0, col = "black")
abline(h = gmean(alpha), v = gmean(beta), col = "blue")

R2.mean <- summary(lm(formula = alpha ~ beta))$"r.squared"
text(x = 0.8*max(beta), y = min(alpha),
     labels = paste("R2.mean = ", signif(R2.mean, digits = 3),sep = ""),
     pos = 3, col = "blue")

for (elt in 1:nbElt) {
  index <- which(mOccur[,elt] == 1)
  points.sd(x = beta[index], y = alpha[index],
            figure = figures[elt], couleur = couleurs[elt],
            nom = colnames(mOccur)[elt], opt.std = FALSE, scale.log = FALSE)
}



#==============================================================================
# Plot all elements with standard deviation WITHOUT RAW DATA WITH SD
#==============================================================================
plot(x = beta,  xlab = "beta",
     y = alpha, ylab = "alpha",
     #     xlim = c(-0.2,0.4), ylim = c(0.4,0.7),
     main = "All elements",
     type = "n", bg = "white", tck = 0.02, las = 1)
abline(h = 0, v = 0, col = "black")
abline(h = gmean(alpha), v = mean(beta), col = "blue")

R2.mean <- summary(lm(formula = alpha ~ beta))$"r.squared"
text(x = 0.5 * max(beta), y = min(alpha),
     labels = paste("R2.mean = ", signif(R2.mean, digits = 3), sep = ""),
     pos = 3, col = "blue")

for (elt in 1:nbElt) {
  index <- which(mOccur[,elt] == 1)
  points.sd(x = beta[index], y = alpha[index],
            figure = figures[elt], couleur = couleurs[elt],
            nom = colnames(mOccur)[elt], opt.std = TRUE, scale.log = FALSE)
}



#==============================================================================
# Plot the hierarchical tree
#==============================================================================
plot(dot.arbre, xlab = "hierarchical tree",
                main = "Agregation Euclidien/Ward", hang = -1, las = 1)




#==============================================================================
# Plot the assembly performances CLUSTERED BY ASSEMBLY MOTIF
#==============================================================================
for (nbclasses in seq_len(nbclassesMax)) {
  # Plot one species
  affectElt <- mAffectElt[nbclasses, ]
  AssMotifs <- mAssMotifs[nbclasses, ]

  setMots   <- sort(unique(AssMotifs))
  nbMots    <- length(setMots)

  talpha    <- test.posthoc(alpha, AssMotifs, pvalue)
  tbeta     <- test.posthoc(beta,  AssMotifs, pvalue)

  noms <- names.assembly(affectElt, mOccur)
  plot.legend(noms)


  # only raw data without either mean neither standard deviation
  plot(x = beta,  xlab = "beta",
       y = alpha, ylab = "alpha",
       type = "n", cex = 2, bg = "white", tck = 0.02, las = 1,
       main = paste("number of classes = ", nbclasses, sep = ""))
  abline(h = 0, v = 0, col = "black")
  abline(h = gmean(alpha), v = mean(beta), col = "blue")

  for (mot in seq_len(nbMots)) {
    motif <- setMots[mot]
    index <- which(AssMotifs == motif)
    points(x = beta[index], y = alpha[index],
           pch = figures[motif], col = couleurs[motif], bg = "white",
           type = "p", cex = 2)
    text(x = mean(beta[index]), y = gmean(alpha[index]),
         col = couleurs[motif],
         labels = noms$motifs[motif], pos = 3, font = 3)
  }


  # without standard deviation
  plot(x = beta,  xlab = "beta",
       y = alpha, ylab = "alpha",
       type = "n", cex = 2, bg = "white", tck = 0.02, las = 1,
       main = paste("number of classes = ", nbclasses, sep = ""))
  abline(h = 0, v = 0, col = "black")
  abline(h = gmean(alpha), v = mean(beta), col = "blue")

  for (mot in seq_len(nbMots)) {
    motif <- setMots[mot]
    index <- which(AssMotifs == motif)
    points(x = beta[index], y = alpha[index],
           pch = figures[motif], col = couleurs[motif], bg = "white",
           type = "p", cex = 2)
    points.sd(x = beta[index], y = alpha[index],
              figure = figures[motif], couleur = couleurs[motif],
              nom = noms$motifs[motif],
              opt.std = FALSE, scale.log = FALSE)
  }


  # with standard deviation
  plot(x = beta,  xlab = "beta",
       y = alpha, ylab = "alpha",
       type = "n", cex = 2, bg = "white", tck = 0.02, las = 1,
       main = paste("number of classes = ", nbclasses, sep = ""))
  abline(h = 0, v = 0, col = "black")
  abline(h = gmean(alpha), v = mean(beta), col = "blue")
  text(x = posX1, y = posY1,labels = "beta", col = "black", font = 3)
  text(x = posX2, y = posY2,labels = "alpha", col = "black", font = 3)

  for (mot in seq_len(nbMots)) {
    motif <- setMots[mot]
    index <- which(AssMotifs == motif)
    points(x = beta[index], y = alpha[index],
           pch = figures[motif], col = couleurs[motif], bg = "white",
           type = "p", cex = 2)
    points.sd(x = beta[index], y = alpha[index],
              figure = figures[motif], couleur = couleurs[motif],
              nom = noms$motifs[motif],
              opt.std = TRUE, scale.log = FALSE)

    if (is.list(talpha)) {
      mta    <- as.matrix(talpha$groups)
      index2 <- which(as.integer(mta[,"trt"]) == motif)
      text(x = posX2, y = as.numeric(mta[index2, "means"]),
           labels = as.character(mta[index2, "M"]),
           col =  couleurs[motif], font = 3)
    }

    if (is.list(tbeta)) {
      mtb    <- as.matrix(tbeta$groups)
      index2 <- which(as.integer(mtb[,"trt"]) == motif)
      text(x = as.numeric(mtb[index2, "means"]), y = posY1,
           labels = as.character(mtb[index2, "M"]),
           col =  couleurs[motif], font = 3)
    }


  }


  # Standard deviation only without raw data
  plot(x = beta,  xlab = "beta",
       y = alpha, ylab = "alpha",
       type = "n", cex = 2, bg = "white", tck = 0.02, las = 1,
       main = paste("number of classes = ", nbclasses, sep = ""))
  abline(h = 0, v = 0, col = "black")
  abline(h = gmean(alpha), v = mean(beta), col = "blue")
  text(x = posX1, y = posY1,labels = "beta", col = "black", font = 3)
  text(x = posX2, y = posY2,labels = "alpha", col = "black", font = 3)

  for (mot in seq_len(nbMots)) {
    motif <- setMots[mot]
    index <- which(AssMotifs == motif)
    points.sd(x = beta[index], y = alpha[index],
              figure = figures[motif], couleur = couleurs[motif],
              nom = noms$motifs[motif],
              opt.std = TRUE, scale.log = FALSE)

    if (is.list(talpha)) {
      mta    <- as.matrix(talpha$groups)
      index2 <- which(as.integer(mta[,"trt"]) == motif)
      text(x = posX2, y = as.numeric(mta[index2, "means"]),
           labels = as.character(mta[index2, "M"]),
           col =  couleurs[motif], font = 3)
    }

    if (is.list(tbeta)) {
      mtb    <- as.matrix(tbeta$groups)
      index2 <- which(as.integer(mtb[,"trt"]) == motif)
      text(x = as.numeric(mtb[index2, "means"]), y = posY1,
           labels = as.character(mtb[index2, "M"]),
           col =  couleurs[motif], font = 3)
    }


  }


  # with Alpha and Beta by motif
  plot(x = beta,  xlab = "beta",
       y = alpha, ylab = "alpha",
       type = "n", cex = 2, bg = "white", tck = 0.02, las = 1,
       main = paste("number of classes = ", nbclasses, sep = ""))
  abline(h = 0, v = 0, col = "black")
  abline(h = gmean(alpha), v = mean(beta), col = "blue")

  for (mot in seq_len(nbMots)) {
    motif <- setMots[mot]
    index <- which(AssMotifs == motif)
    points(x = beta[index], y = alpha[index],
           pch = figures[mot], col = couleurs[mot], bg = "white",
           type = "p", cex = 2)

    for (elt in seq_len(nbElt)) {
      index2 <- which(mOccur[index,elt] == 1)
      titre  <- paste(noms$motifs[motif],
                      colnames(mOccur)[elt], sep = ".")
      points.sd(x = beta[index[index2]], y = alpha[index[index2]],
                figure = figures[elt], couleur = couleurs[mot],
                nom = titre,
                opt.std = FALSE, scale.log = FALSE)
    }
  }
}



#==============================================================================
# Plot the assembly performances clustered by the NUMBER of SPECIES
#==============================================================================
if (opt.size == TRUE) {

  mainTitre <- "Assembly size"

  size      <- apply(mOccur, MARGIN = 1, FUN = sum)
  AssMotifs <- numeric(length(size))
  for (i in seq_along(unique(size))) AssMotifs[(size == unique(size)[i])] <- i

  setMots <- sort(unique(AssMotifs))
  nbMots  <- length(setMots)
  noms    <- as.character(sort(unique(size)))

  talpha <- test.posthoc(alpha, AssMotifs, pvalue)
  tbeta  <- test.posthoc(beta,  AssMotifs, pvalue)

  #----------------------------------------------------------------------------
  # without standard deviation
  plot(x = beta,  xlab = "beta",
       y = alpha, ylab = "alpha",
       main = mainTitre,
       type = "n", cex = 2, bg = "white", tck = 0.02, las = 1)
  abline(h = 0, v = 0, col = "black")
  abline(h = gmean(alpha), v = mean(beta), col = "blue")

  for (mot in seq_len(nbMots)) {
    motif <- setMots[mot]
    index <- which(AssMotifs == motif)
    points(x = beta[index], y = alpha[index],
           pch = figures[motif], col = couleurs[motif],
           type = "p", bg = "white", cex = 2)
    points.sd(x = beta[index], y = alpha[index],
              figure = figures[motif], couleur = couleurs[motif],
              nom = noms[motif],
              opt.std = FALSE, scale.log = FALSE)
  }


  # with standard deviation
  plot(x = beta,  xlab = "beta",
       y = alpha, ylab = "alpha",
       main = mainTitre,
       type = "n", cex = 2, bg = "white", tck = 0.02, las = 1)
  abline(h = 0, v = 0, col = "black")
  abline(h = gmean(alpha), v = mean(beta), col = "blue")
  text(x = posX1, y = posY1, labels = "beta", col = "black", font = 3)
  text(x = posX2, y = posY2, labels = "alpha", col = "black", font = 3)

  for (mot in seq_len(nbMots)) {
    motif <- setMots[mot]
    index <- which(AssMotifs == motif)
    points(x = beta[index], y = alpha[index],
           pch = figures[motif], col = couleurs[motif], bg = "white",
           type = "p", cex = 2)
    points.sd(x = beta[index], y = alpha[index],
              figure = figures[motif], couleur = couleurs[motif],
              nom = noms[motif],
              opt.std = TRUE, scale.log = FALSE)

    if (is.list(talpha)) {
      mta    <- as.matrix(talpha$groups)
      index2 <- which(as.integer(mta[,"trt"]) == motif)
      text(x = posX2, y = as.numeric(mta[index2, "means"]),
           labels = as.character(mta[index2, "M"]),
           col =  couleurs[motif], font = 3)
    }

    if (is.list(tbeta)) {
      mtb    <- as.matrix(tbeta$groups)
      index2 <- which(as.integer(mtb[,"trt"]) == motif)
      text(x = as.numeric(mtb[index2, "means"]), y = posY1,
           labels = as.character(mtb[index2, "M"]),
           col =  couleurs[motif], font = 3)
    }
  }
}



#==============================================================================
# Plot the assembly performances clustered by A PRIORI FUNCTIONAL GROUPS
#==============================================================================

if (opt.gf == TRUE) {

  mainTitre <- "Functional Groups"

  affectElt <- gf
  names(affectElt) <- colnames(mOccur)

  AssMotifs <- affect.motifs(affectElt, mOccur)
  setMots   <- sort(unique(AssMotifs))
  nbMots    <- length(setMots)

  talpha <- test.posthoc(alpha, AssMotifs, pvalue)
  tbeta  <- test.posthoc(beta,  AssMotifs, pvalue)

  noms <- names.assembly(affectElt, mOccur)
  plot.legend(noms)


  #----------------------------------------------------------------------------
  # without standard deviation
  plot(x = beta,  xlab = "beta",
       y = alpha, ylab = "alpha",
       main = mainTitre,
       type = "n", bg = "white", cex = 2, tck = 0.02, las = 1)
  abline(h = 0, v = 0, col = "black")
  abline(h = gmean(alpha), v = mean(beta), col = "blue")

  for (mot in seq_len(nbMots)) {
    motif <- setMots[mot]
    index <- which(AssMotifs == motif)
    points(x = beta[index], y = alpha[index],
           pch = figures[motif], col = couleurs[motif], bg = "white",
           type = "p", cex = 2)
    points.sd(x = beta[index], y = alpha[index],
              figure = figures[motif], couleur = couleurs[motif],
              nom = noms$motifs[motif],
              opt.std = FALSE, scale.log = FALSE)
  }


  # with standard deviation
  plot(x = beta,  xlab = "beta",
       y = alpha, ylab = "alpha",
       main = mainTitre,
       type = "n", cex = 2, bg = "white", tck = 0.02, las = 1)
  abline(h = 0, v = 0, col = "black")
  abline(h = gmean(alpha), v = mean(beta), col = "blue")
  text(x = posX1, y = posY1,labels = "beta", col = "black", font = 3)
  text(x = posX2, y = posY2,labels = "alpha", col = "black", font = 3)

  for (mot in seq_len(nbMots)) {
    motif <- setMots[mot]
    index <- which(AssMotifs == motif)
    points(x = beta[index], y = alpha[index],
           pch = figures[motif], col = couleurs[motif], bg = "white",
           type = "p", cex = 2)
    points.sd(x = beta[index], y = alpha[index],
              figure = figures[motif], couleur = couleurs[motif],
              nom = noms$motifs[motif],
              opt.std = TRUE, scale.log = FALSE)

    if (is.list(talpha)) {
      mta    <- as.matrix(talpha$groups)
      index2 <- which(as.integer(mta[,"trt"]) == motif)
      text(x = posX2, y = as.numeric(mta[index2, "means"]),
           labels = as.character(mta[index2, "M"]),
           col =  couleurs[motif], font = 3)
    }

    if (is.list(tbeta)) {
      mtb    <- as.matrix(tbeta$groups)
      index2 <- which(as.integer(mtb[,"trt"]) == motif)
      text(x = as.numeric(mtb[index2, "means"]), y = posY1,
           labels = as.character(mtb[index2, "M"]),
           col =  couleurs[motif], font = 3)
    }
  }
}



#==============================================================================
#==============================================================================
# SCALE LOG == TRUE  (opt.log == TRUE)
#==============================================================================
#==============================================================================

if (opt.log == TRUE) {


#==============================================================================
# Parameters for graph plotting
#==============================================================================
loghigth <- max(log2(alpha)) - min(log2(alpha))
logwidth <- max(log2(beta))  - min(log2(beta))
logposX1 <- min(log2(beta))  - 0.02*logwidth
logposX2 <- min(log2(beta))  + 1.02*logwidth
logposY1 <- min(log2(alpha)) - 0.02*loghigth
logposY2 <- min(log2(alpha)) + 1.02*loghigth



#==============================================================================
# Plot alpha
#==============================================================================
titre <- "alpha"

ghisto(alpha, titre, scale.log = TRUE)
plot.alphaVSbeta(x = fobs,  xlab = "fobs",
                 y = alpha, ylab = titre,
                 titre = titre, scale.log = TRUE)

plot.alphaVSbeta(y = fobs,  ylab = "fobs",
                 x = alpha, xlab = titre,
                 titre = titre, scale.log = TRUE)



#==============================================================================
# Plot beta
#==============================================================================
titre <- "beta"

ghisto(beta, titre, scale.log = TRUE)
plot.alphaVSbeta(x = fobs, xlab = "fobs",
                 y = beta, ylab = titre,
                 titre = titre, scale.log = TRUE)

plot.alphaVSbeta(y = fobs, ylab = "fobs",
                 x = beta, xlab = titre,
                 titre = titre, scale.log = TRUE)



#==============================================================================
# Plot log2(alpha) vs log2(beta)
#==============================================================================
plot.alphaVSbeta(x = beta,  xlab = "beta",
                 y = alpha, ylab = "alpha",
                 titre = "alpha vs beta", scale.log = TRUE)



#==============================================================================
# Plot log2(alpha) vs log2(beta) WITH TRANSGRESSIVE ASSEMBLAGES in RED
#==============================================================================
talpha <- test.posthoc(alpha, trans, pvalue)
tbeta  <- test.posthoc(beta,  trans, pvalue)
setNoms <- c("trans-under", "under", "over", "trans-over")

plot(x = log2(beta),  xlab = "log2(beta)",
     y = log2(alpha), ylab = "log2(alpha)",
     main = "log2(alpha) vs log2(beta): transgressive overyielding",
     type = "n", tck = 0.02, las = 1)

abline(v = log2(mean(beta)), h = log2(mean(alpha)), col = "blue")
text(x = logposX1, y = logposY1,labels = "beta", col = "black", font = 3)
text(x = logposX2, y = logposY2,labels = "alpha", col = "black", font = 3)

setTrans <- sort(unique(trans), decreasing = TRUE)
for (i in 1:length(setTrans)) {
  j <- setTrans[i]
  points(x = log2(beta[(trans == j)]), y = log2(alpha[(trans == j)]),
         type = "p", pch = figures[i], col = couleurs[i],
         bg = "white", cex = 2)

  points.sd(x = beta[(trans == j)],  y = alpha[(trans == j)],
            figure = figures[i], couleur = couleurs[i], nom = setNoms[j],
            opt.std = TRUE, scale.log = TRUE)

  if (is.list(talpha)) {
    mta   <- as.matrix(talpha$groups)
    index <- which(as.integer(mta[,"trt"]) == j)
    text(x = logposX2, y = log2(as.numeric(mta[index, "means"])),
         labels = as.character(mta[index, "M"]),
         col =  couleurs[i], font = 3)
  }

  if (is.list(tbeta)) {
    mtb   <- as.matrix(tbeta$groups)
    index <- which(as.integer(mtb[,"trt"]) == j)
    text(x = log2(as.numeric(mtb[index, "means"])), y = logposY1,
         labels = as.character(mtb[index, "M"]),
         col =  couleurs[i], font = 3)
  }
}



#==============================================================================
# Plot the mean performance of assemblages containing a given element in LOG
#==============================================================================
for (elt in seq_len(nbElt)) {
  index <- which(mOccur[,elt] == 1)

  plot(x = log2(beta),  xlab = "log2(beta)",
       y = log2(alpha), ylab = "log2(alpha)",
       main = paste("Element = ", colnames(mOccur)[elt], sep = ""),
       type = "n", tck = 0.02, las = 1)
  points(x = log2(beta[-index]), y = log2(alpha[-index]),
         type = "p", pch = figures[1], col = "black", bg = "white", cex = 2)
  points(x = log2(beta[index]), y = log2(alpha[index]),
         type = "p", pch = figures[2], col = "blue", bg = "white", cex = 2)
  points.sd(x = beta[index], y = alpha[index],
            figure = figures[2], couleur = "blue", nom = colnames(mOccur)[elt],
            opt.std = TRUE, scale.log = TRUE)
  abline(h = 0, v = 0, col = "black")
  abline(h = log2(gmean(alpha)), v = log2(mean(beta)), col = "blue")
}



#==============================================================================
# Plot all species without standard deviation WITH RAW DATA  IN LOG
#==============================================================================
plot(x = log2(beta),  xlab = "log2(beta)",
     y = log2(alpha), ylab = "log2(alpha)",
     main = "All elements",
     pch = figures[1], col = "black", bg = "white",
     type = "p", cex = 2, tck = 0.02, las = 1)

abline(h = 0, v = 0, col = "black")
abline(h = log2(gmean(alpha)), v = log2(mean(beta)), col = "blue")

for (elt in 1:nbElt) {
  index <- which(mOccur[,elt] == 1)
  points.sd(x = beta[index], y = alpha[index],
            figure = figures[elt], couleur = couleurs[elt],
            nom = colnames(mOccur)[elt], opt.std = FALSE, scale.log = TRUE)
}



#==============================================================================
# Plot all species without standard deviation WITH RAW DATA WITH SD  IN LOG
#==============================================================================
plot(x = log2(beta),  xlab = "log2(beta)",
     y = log2(alpha), ylab = "log2(alpha)",
     main = "All elements",
     pch = figures[1], col = "black", bg = "white",
     type = "p", cex = 2, tck = 0.02, las = 1)

abline(h = 0, v = 0, col = "black")
abline(h = log2(gmean(alpha)), v = log2(mean(beta)), col = "blue")

for (elt in 1:nbElt) {
  index <- which(mOccur[,elt] == 1)
  points.sd(x = log2(beta[index]), y = log2(alpha[index]),
            figure = figures[elt], couleur = couleurs[elt],
            nom = colnames(mOccur)[elt], opt.std = TRUE, scale.log = FALSE)
}



#==============================================================================
# Plot all species without standard deviation WITHOUT RAW DATA  IN LOG
#==============================================================================
plot(x = log2(beta),  xlab = "log2(beta)",
     y = log2(alpha), ylab = "log2(alpha)",
     #     xlim = c(-0.2,0.4), ylim = c(0.4,0.7),
     main = "All elements",
     type = "n", bg = "white", tck = 0.02, las = 1)
abline(h = 0, v = 0, col = "black")
abline(h = log2(gmean(alpha)), v = log2(gmean(beta)), col = "blue")

R2.mean <- summary(lm(formula = alpha ~ beta))$"r.squared"
text(x = 0.8*max(log2(beta)), y = min(log2(alpha)),
     labels = paste("R2.mean = ", signif(R2.mean, digits = 3),sep = ""),
     pos = 3, col = "blue")

for (elt in 1:nbElt) {
  index <- which(mOccur[,elt] == 1)
  points.sd(x = beta[index], y = alpha[index],
            figure = figures[elt], couleur = couleurs[elt],
            nom = colnames(mOccur)[elt], opt.std = FALSE, scale.log = TRUE)
}



#==============================================================================
# Plot all elements with standard deviation WITHOUT RAW DATA WITH SD IN LOG
#==============================================================================
plot(x = log2(beta),  xlab = "log2(beta)",
     y = log2(alpha), ylab = "log2(alpha)",
     #     xlim = c(-0.2,0.4), ylim = c(0.4,0.7),
     main = "All elements",
     type = "n", bg = "white", tck = 0.02, las = 1)
abline(h = 0, v = 0, col = "black")
abline(h = log2(gmean(alpha)), v = log2(gmean(beta)), col = "blue")

R2.mean <- summary(lm(formula = alpha ~ beta))$"r.squared"
text(x = 0.5 * max(log2(beta)), y = min(log2(alpha)),
     labels = paste("R2.mean = ", signif(R2.mean, digits = 3), sep = ""),
     pos = 3, col = "blue")

for (elt in 1:nbElt) {
  index <- which(mOccur[,elt] == 1)
  points.sd(x = beta[index], y = alpha[index],
            figure = figures[elt], couleur = couleurs[elt],
            nom = colnames(mOccur)[elt], opt.std = TRUE, scale.log = TRUE)
}



#==============================================================================
# Plot the hierarchical tree
#==============================================================================
plot(dot.arbre, xlab = "hierarchical tree",
     main = "Agregation Euclidien/Ward", hang = -1, las = 1)




#==============================================================================
# Plot the assembly performances CLUSTERED BY ASSEMBLY MOTIF  IN LOG
#==============================================================================
for (nbclasses in seq_len(nbclassesMax)) {
  # Plot one species
  affectElt <- mAffectElt[nbclasses, ]
  AssMotifs <- mAssMotifs[nbclasses, ]

  setMots   <- sort(unique(AssMotifs))
  nbMots    <- length(setMots)

  talpha <- test.posthoc(alpha, AssMotifs, pvalue)
  tbeta  <- test.posthoc(beta,  AssMotifs, pvalue)

  noms <- names.assembly(affectElt, mOccur)
  plot.legend(noms)


  # only raw data without either mean neither standard deviation
  plot(x = log2(beta),  xlab = "log2(beta)",
       y = log2(alpha), ylab = "log2(alpha)",
       type = "n", cex = 2, bg = "white", tck = 0.02, las = 1,
       main = paste("number of classes = ", nbclasses, sep = ""))
  abline(h = 0, v = 0, col = "black")
  abline(h = log2(gmean(alpha)), v = log2(gmean(beta)), col = "blue")

  for (mot in seq_len(nbMots)) {
    motif <- setMots[mot]
    index <- which(AssMotifs == motif)
    points(x = log2(beta[index]), y = log2(alpha[index]),
           pch = figures[motif], col = couleurs[motif], bg = "white",
           type = "p", cex = 2)
    text(x = log2(gmean(beta[index])), y = log2(gmean(alpha[index])),
         col = couleurs[motif],
         labels = noms$motifs[motif], pos = 3, font = 3)
  }


  # without standard deviation
  plot(x = log2(beta),  xlab = "log2(beta)",
       y = log2(alpha), ylab = "log2(alpha)",
       type = "n", cex = 2, bg = "white", tck = 0.02, las = 1,
       main = paste("number of classes = ", nbclasses, sep = ""))
  abline(h = 0, v = 0, col = "black")
  abline(h = log2(gmean(alpha)), v = log2(gmean(beta)), col = "blue")

  for (mot in seq_len(nbMots)) {
    motif <- setMots[mot]
    index <- which(AssMotifs == motif)
    points(x = log2(beta[index]), y = log2(alpha[index]),
           pch = figures[motif], col = couleurs[motif], bg = "white",
           type = "p", cex = 2)
    points.sd(x = beta[index], y = alpha[index],
              figure = figures[motif], couleur = couleurs[motif],
              nom = noms$motifs[motif],
              opt.std = FALSE, scale.log = TRUE)
  }


  # with standard deviation
  plot(x = log2(beta),  xlab = "log2(beta)",
       y = log2(alpha), ylab = "log2(alpha)",
       type = "n", cex = 2, bg = "white", tck = 0.02, las = 1,
       main = paste("number of classes = ", nbclasses, sep = ""))
  abline(h = 0, v = 0, col = "black")
  abline(h = log2(gmean(alpha)), v = log2(gmean(beta)), col = "blue")
  text(x = logposX1, y = logposY1,labels = "beta", col = "black", font = 3)
  text(x = logposX2, y = logposY2,labels = "alpha", col = "black", font = 3)

  for (mot in seq_len(nbMots)) {
    motif <- setMots[mot]
    index <- which(AssMotifs == motif)
    points(x = log2(beta[index]), y = log2(alpha[index]),
           pch = figures[motif], col = couleurs[motif], bg = "white",
           type = "p", cex = 2)
    points.sd(x = beta[index], y = alpha[index],
              figure = figures[motif], couleur = couleurs[motif],
              nom = noms$motifs[motif],
              opt.std = TRUE, scale.log = TRUE)

    if (is.list(talpha)) {
      mta    <- as.matrix(talpha$groups)
      index2 <- which(as.integer(mta[,"trt"]) == motif)
      text(x = logposX2, y = log2(as.numeric(mta[index2, "means"])),
           labels = as.character(mta[index2, "M"]),
           col =  couleurs[motif], font = 3)
    }

    if (is.list(tbeta))
    {
      mtb    <- as.matrix(tbeta$groups)
      index2 <- which(as.integer(mtb[,"trt"]) == motif)
      text(x = log2(as.numeric(mtb[index2, "means"])), y = logposY1,
           labels = as.character(mtb[index2, "M"]),
           col =  couleurs[motif], font = 3)
    }


  }


  # with Alpha and Beta by motif
  plot(x = log2(beta),  xlab = "log2(beta)",
       y = log2(alpha), ylab = "log2(alpha)",
       type = "n", cex = 2, bg = "white", tck = 0.02, las = 1,
       main = paste("number of classes = ", nbclasses, sep = ""))
  abline(h = 0, v = 0, col = "black")
  abline(h = log2(gmean(alpha)), v = log2(gmean(beta)), col = "blue")

  for (mot in seq_len(nbMots)) {
    motif <- setMots[mot]
    index <- which(AssMotifs == motif)
    points(x = log2(beta[index]), y = log2(alpha[index]),
           pch = figures[mot], col = couleurs[mot], bg = "white",
           type = "p", cex = 2)

    for (elt in seq_len(nbElt)) {
      index2 <- which(mOccur[index,elt] == 1)
      titre  <- paste(noms$motifs[motif],
                      colnames(mOccur)[elt], sep = ".")
      points.sd(x = beta[index[index2]], y = alpha[index[index2]],
                figure = figures[elt], couleur = couleurs[mot],
                nom = titre,
                opt.std = FALSE, scale.log = TRUE)
    }
  }
}



#==============================================================================
# Plot the assembly performances clustered by the NUMBER of SPECIES
#==============================================================================
if (opt.size == TRUE) {

  mainTitre <- "Assembly size"

  size      <- apply(mOccur, MARGIN = 1, FUN = sum)
  AssMotifs <- numeric(length(size))
  for (i in seq_along(unique(size))) AssMotifs[(size == unique(size)[i])] <- i

  setMots <- sort(unique(AssMotifs))
  nbMots  <- length(setMots)
  noms    <- as.character(sort(unique(size)))

  talpha <- test.posthoc(alpha, AssMotifs, pvalue)
  tbeta  <- test.posthoc(beta,  AssMotifs, pvalue)

  #----------------------------------------------------------------------------
  # without standard deviation
  plot(x = log2(beta),  xlab = "log2(beta)",
       y = log2(alpha), ylab = "log2(alpha)",
       main = mainTitre,
       type = "n", cex = 2, bg = "white", tck = 0.02, las = 1)
  abline(h = 0, v = 0, col = "black")
  abline(h = log2(gmean(alpha)), v = log2(gmean(beta)), col = "blue")

  for (mot in 1:nbMots) {
    motif <- setMots[mot]
    index <- which(AssMotifs == motif)
    points(x = log2(beta[index]), y = log2(alpha[index]),
           pch = figures[motif], col = couleurs[motif],
           type = "p", bg = "white", cex = 2)
    points.sd(x = beta[index], y = alpha[index],
              figure = figures[motif], couleur = couleurs[motif],
              nom = noms[motif],
              opt.std = FALSE, scale.log = TRUE)
  }


  # with standard deviation
  plot(x = log2(beta),  xlab = "log2(beta)",
       y = log2(alpha), ylab = "log2(alpha)",
       main = mainTitre,
       type = "n", cex = 2, bg = "white", tck = 0.02, las = 1)
  abline(h = 0, v = 0, col = "black")
  abline(h = log2(gmean(alpha)), v = log2(gmean(beta)), col = "blue")
  text(x = logposX1, y = logposY1,labels = "beta", col = "black", font = 3)
  text(x = logposX2, y = logposY2,labels = "alpha", col = "black", font = 3)

  for (mot in seq_len(nbMots)) {
    motif <- setMots[mot]
    index <- which(AssMotifs == motif)
    points(x = log2(beta[index]), y = log2(alpha[index]),
           pch = figures[motif], col = couleurs[motif], bg = "white",
           type = "p", cex = 2)
    points.sd(x = beta[index], y = alpha[index],
              figure = figures[motif], couleur = couleurs[motif],
              nom = noms[motif],
              opt.std = TRUE, scale.log = TRUE)

    if (is.list(talpha)) {
      mta    <- as.matrix(talpha$groups)
      index2 <- which(as.integer(mta[,"trt"]) == motif)
      text(x = logposX2, y = log2(as.numeric(mta[index2, "means"])),
           labels = as.character(mta[index2, "M"]),
           col =  couleurs[motif], font = 3)
    }

    if (is.list(tbeta)) {
      mtb    <- as.matrix(tbeta$groups)
      index2 <- which(as.integer(mtb[,"trt"]) == motif)
      text(x = log2(as.numeric(mtb[index2, "means"])), y = logposY1,
           labels = as.character(mtb[index2, "M"]),
           col =  couleurs[motif], font = 3)
    }
  }
}


#==============================================================================
# Plot the assembly performances clustered by A PRIORI FUNCTIONAL GROUPS
#==============================================================================

if (opt.gf == TRUE) {

  mainTitre <- "Functional Groups"

  affectElt <- gf
  AssMotifs <- affect.motifs(affectElt, mOccur)
  setMots   <- sort(unique(AssMotifs))
  nbMots    <- length(setMots)

  talpha <- test.posthoc(alpha, AssMotifs, pvalue)
  tbeta  <- test.posthoc(beta,  AssMotifs, pvalue)

  noms <- names.assembly(affectElt, mOccur)
  plot.legend(noms)


  #----------------------------------------------------------------------------
  # without standard deviation
  plot(x = log2(beta),  xlab = "log2(beta)",
       y = log2(alpha), ylab = "log2(alpha)",
       main = mainTitre,
       type = "n", bg = "white", cex = 2, tck = 0.02, las = 1)
  abline(h = 0, v = 0, col = "black")
  abline(h = log2(gmean(alpha)), v = log2(gmean(beta)), col = "blue")

  for (mot in seq_len(nbMots)) {
    motif <- setMots[mot]
    index <- which(AssMotifs == motif)
    points(x = log2(beta[index]), y = log2(alpha[index]),
           pch = figures[motif], col = couleurs[motif], bg = "white",
           type = "p", cex = 2)
    points.sd(x = beta[index], y = alpha[index],
              figure = figures[motif], couleur = couleurs[motif],
              nom = noms$motifs[motif],
              opt.std = FALSE, scale.log = TRUE)
  }


  # with standard deviation
  plot(x = log2(beta),  xlab = "log2(beta)",
       y = log2(alpha), ylab = "log2(alpha)",
       main = mainTitre,
       type = "n", cex = 2, bg = "white", tck = 0.02, las = 1)
  abline(h = 0, v = 0, col = "black")
  abline(h = log2(gmean(alpha)), v = log2(gmean(beta)), col = "blue")
  text(x = logposX1, y = logposY1,labels = "beta", col = "black", font = 3)
  text(x = logposX2, y = logposY2,labels = "alpha", col = "black", font = 3)

  for (mot in seq_len(nbMots)) {
    motif <- setMots[mot]
    index <- which(AssMotifs == motif)
    points(x = log2(beta[index]), y = log2(alpha[index]),
           pch = figures[motif], col = couleurs[motif], bg = "white",
           type = "p", cex = 2)
    points.sd(x = beta[index], y = alpha[index],
              figure = figures[motif], couleur = couleurs[motif],
              nom = noms$motifs[motif],
              opt.std = TRUE, scale.log = TRUE)

    if (is.list(talpha)) {
      mta    <- as.matrix(talpha$groups)
      index2 <- which(as.integer(mta[,"trt"]) == motif)
      text(x = logposX2, y = log2(as.numeric(mta[index2, "means"])),
           labels = as.character(mta[index2, "M"]),
           col =  couleurs[motif], font = 3)
    }

    if (is.list(tbeta)) {
      mtb    <- as.matrix(tbeta$groups)
      index2 <- which(as.integer(mtb[,"trt"]) == motif)
      text(x = log2(as.numeric(mtb[index2, "means"])), y = logposY1,
           labels = as.character(mtb[index2, "M"]),
           col =  couleurs[motif], font = 3)
    }
  }
}

#==============================================================================
#==============================================================================
# END of SCALE LOG == TRUE  (opt.log == TRUE)
#==============================================================================
#==============================================================================
}


#==============================================================================
#==============================================================================
# END of SCRIPT
#==============================================================================
#==============================================================================


